import { Component, OnInit, ViewChild, Inject, OnDestroy } from '@angular/core'
import { ProviderId, UserConsent, UserConsentVerbs } from './consentDeclaration'
import { MatAccordion } from '@angular/material/expansion'
import { ApiService, Provider } from '../services/api.service'
import { ToastrService } from 'ngx-toastr'
import { MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog'
import { DataRemovalService } from '../data-removal.service'
import { DeleteDialog } from '../dialogs/delete-dialog/delete-dialog'
import { Subscription, zip } from 'rxjs'
import { extractVerbs } from './wizard/wizard.component'


@Component({
    selector: 'app-consent-management',
    templateUrl: './consent-management.component.html',
    styleUrls: ['./consent-management.component.scss']
})
export class ConsentManagementComponent implements OnInit, OnDestroy {
    @ViewChild(MatAccordion) accordion!: MatAccordion
    providers: Provider[] = []
    providerUserConsent: Record<ProviderId, UserConsent | null> = {}
    userConsents: UserConsentVerbs[] = []
    pauseDialogSub?: Subscription
    pausedDataRecording: boolean = false

    constructor(
        private _apiService: ApiService,
        private _removalService: DataRemovalService,
        private _toasterService: ToastrService,
        private _dialog: MatDialog
    ) {}

    ngOnInit(): void {
        this.getProviderUserConsents()
    }

    getProviderUserConsents() {
        this._apiService.getProviders().subscribe((providers) => {
            this.providers = providers

            const userConsents$ = zip(
                providers.map((provider) => this._apiService.getUserConsent(provider.id))
            )
            userConsents$.subscribe((userConsents) => {
                this.providerUserConsent = userConsents.reduce((acc, userConsent, idx) => {
                    acc[providers[idx].id] = userConsent.consent
                    return acc
                }, {} as Record<ProviderId, UserConsent | null>)

                this.userConsents = providers.map((provider) => {
                    const userConsent = this.providerUserConsent[provider.id]!
                    return {
                        providerId: provider.id,
                        providerSchemaId: provider.versions[0].id,
                        verbs: extractVerbs(userConsent, provider.id)
                    }
                })

                this.pausedDataRecording = !!userConsents.find((e) => !!e.paused_data_recording)
            })
        })
    }

    ngOnDestroy(): void {
        this.pauseDialogSub?.unsubscribe()
    }

    scroll(selector: string) {
        document.getElementById(selector)?.scrollIntoView({
            behavior: 'smooth',
            block: 'start',
            inline: 'nearest'
        })
    }

    changePrivacySetting(userConsent: UserConsent, provider: Provider, providerIdx: number) {
      //  this.userConsents[providerIdx].verbs = extractVerbs(userConsent, provider.id)
      let consent = this.userConsents?.find(ele => ele.providerId == provider.id)
      if(consent)
          consent.verbs = extractVerbs(userConsent, provider.id)
    }

    saveChanges(): void {
        this._apiService.saveUserConsent(this.userConsents).subscribe((response) => {
            this._toasterService.success($localize`:@@toastMessageSaved:Saved`)
            this.getProviderUserConsents()
        })
    }

    showPauseDialog(): void {
        this._dialog.open(PauseDataRecordingDialog, {
            width: '400px',
            data: {
                isPaused: this.pausedDataRecording,
                toggleDataRecording: () => {
                    this._apiService.toggleDataRecording().subscribe((response) => {
                        this._toasterService.success($localize`:@@toastMessageChanged:Changed`)
                        this.getProviderUserConsents()
                    })
                },
                pauseDataRecordingDeleteData: () => {
                    const dialogRef = this._dialog.open(DeleteDialog, {
                        data: {
                            headerText: $localize`:@@pauseRecordingAndDeleteData:Pause and Delete Confirmation`
                        }
                    })

                    this.pauseDialogSub = dialogRef.afterClosed().subscribe((confirmed) => {
                        if (confirmed)
                            this._apiService.toggleDataRecording().subscribe((response) => {
                                this._removalService
                                    .create({ immedialty: false, scope: {} })
                                    .subscribe((response) => {
                                        this._toasterService.success(
                                            $localize`:@@toastMessageChanged:Changed`
                                        )
                                        this.getProviderUserConsents()
                                    })
                            })
                    })
                }
            }
        })
    }
}

export interface DialogData {
    isPaused: boolean
    toggleDataRecording: () => void
    pauseDataRecordingDeleteData: () => void
}

@Component({
    selector: 'pause-data-recording-dialog',
    templateUrl: 'pause-data-recording-dialog.html'
})
export class PauseDataRecordingDialog {
    constructor(@Inject(MAT_DIALOG_DATA) public data: DialogData) {}
}
