import { HttpClient, HttpEventType } from '@angular/common/http'
import { Component, OnInit } from '@angular/core'
import { Subscription } from 'rxjs'
import { environment } from '../../../environments/environment'
import { ApiService, Provider } from 'src/app/services/api.service'
import { ToastrService } from 'ngx-toastr'
import {
    ConsentGroupConsented,
    ProviderSchema,
    providerSchemaToUserConsent,
    UserConsent
} from '../consentDeclaration'

type ProviderId = string
type ProviderSchemaVersionId = string

@Component({
    selector: 'app-provider',
    templateUrl: './provider.component.html',
    styleUrls: ['./provider.component.scss']
})
export class ProviderComponent implements OnInit {
    providers: Provider[] = []
    fileName = $localize`:@@selectProviderSchema:Please select a JSON file.`
    selectedFile?: File
    uploadProgress: number | null = null
    uploadSub: Subscription | null = null
    definitionVisible: Record<ProviderId, Record<ProviderSchemaVersionId, boolean>> = {}

    constructor(
        private _http: HttpClient,
        private _apiService: ApiService,
        private _toasterService: ToastrService
    ) {}

    ngOnInit(): void {
        this.getAllProviders()
    }

    getAllProviders() {
        this._apiService.getProviders().subscribe((providers) => {
            this.providers = providers

            providers.map((provider) => {
                const res = provider.versions.reduce((acc, e) => {
                    {
                        acc[e.id] = false
                    }
                    return acc
                }, {} as Record<ProviderSchemaVersionId, boolean>)
                this.definitionVisible[provider.id] = res
            })
        })
    }

    onFileSelected(event: any) {
        const file: File = event.target.files[0]

        if (file) {
            this.selectedFile = file
            this.fileName = file.name
        }
    }

    onSubmit() {
        if (this.selectedFile) {
            const formData = new FormData()
            formData.append('filename', this.fileName)
            formData.append('provider-schema', this.selectedFile)

            const upload$ = this._http.put(
                `${environment.apiUrl}/api/v1/consents/provider/create`,
                formData,
                {
                    reportProgress: true,
                    observe: 'events'
                }
            )

            this.uploadSub = upload$.subscribe({
                next: (event) => {
                    if (event.type == HttpEventType.UploadProgress) {
                        this.uploadProgress = Math.round(100 * (event.loaded / (event.total ?? 1)))
                    }
                },
                complete: () => {
                    this.reset()
                    this.getAllProviders()
                    this._toasterService.success($localize`:@@toastMessageCreated:Created`)
                },
                error: () => {
                    // Error toast is triggered via interceptor
                    this.reset()
                }
            })
        }
    }

    cancelUpload() {
        this.uploadSub?.unsubscribe()
        this.reset()
    }

    reset() {
        this.fileName = $localize`:@@selectProviderSchema:Please select a JSON file.`
        this.uploadProgress = null
        this.uploadSub = null
        window.scroll(0, 0)
    }

    providerSchemaToUserConsent(providerSchema: ProviderSchema): UserConsent {
        return providerSchemaToUserConsent(providerSchema)
    }
}
