import { Component, Inject } from '@angular/core'
import {
    UserConsent,
    providerSchemaToUserConsent,
    ProviderSchema,
    ProviderId,
    UserConsentVerbs
} from '../consentDeclaration'
import { ApiService, Provider, UserConsentStatus } from 'src/app/services/api.service'
import { ToastrService } from 'ngx-toastr'
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog'
import { AuthService } from 'src/app/services/auth.service'
import { zip } from 'rxjs'

export const extractVerbs = (providerSchema: UserConsent, providerId: ProviderId): any[] => {
    return providerSchema.groups.flatMap((group) => {
        return group.verbs.flatMap(({ id, consented, defaultConsent, objects }) => ({
            provider: providerId,
            id,
            consented: consented,
            objects: objects ? JSON.stringify(
                objects.map((object) => ({
                    ...object,
                    consented: object.consented
                }))
            ) : []
        }))
    })
}

@Component({
    selector: 'wizard-dialog',
    templateUrl: 'wizard.component.html',
    styleUrls: ['./wizard.component.scss']
})
export class WizardDialog {
    constructor(
        public dialogRef: MatDialogRef<WizardDialog>,
        @Inject(MAT_DIALOG_DATA) public data: UserConsentStatus | null,
        private _apiService: ApiService,
        private _toasterService: ToastrService,
        private _authService: AuthService
    ) {}

    loading = false
    providers: Provider[] = []
    providerIds: number[] = []
    providerUserConsent: Record<
        ProviderId,
        {
            provider: Provider
            userConsent: UserConsent | null
            defaultConsent: UserConsent
            acceptedProviderSchema?: ProviderSchema
        }
    > = {}
    wizardUserConsents: Record<
        ProviderId,
        {
            provider: Provider
            userConsent: UserConsent | null
            defaultConsent: UserConsent
            acceptedProviderSchema?: ProviderSchema
        }
    > = {}
    userConsents: UserConsentVerbs[] = []
    selectedProviderId: number | null = null
    isSummary = false

    ngOnInit(): void {
        this.loading = true
        this._apiService.getProviders().subscribe((providers) => {
            this.providers = providers

            if (providers.length === 0) this.loading = false

            const userConsents$ = zip(
                providers.map((provider) => this._apiService.getUserConsent(provider.id))
            )

            userConsents$.subscribe((userConsents) => {
                this.providerUserConsent = userConsents.reduce((acc, userConsent, idx) => {
                    acc[providers[idx].id] = {
                        provider: providers[idx],
                        userConsent: userConsent.consent,
                        acceptedProviderSchema: userConsent.provider_schema,
                        defaultConsent: providerSchemaToUserConsent(providers[idx].versions[0])
                    }

                    return acc
                }, {} as Record<ProviderId, { provider: Provider; userConsent: UserConsent | null; defaultConsent: UserConsent; acceptedProviderSchema?: ProviderSchema }>)

                this.wizardUserConsents = userConsents.reduce((acc, userConsent, idx) => {
                    // Skip providers, for which user already accepted the latest version
                    if (userConsent.provider_schema?.superseded_by === null) return acc
                    acc[providers[idx].id] = {
                        provider: providers[idx],
                        userConsent: userConsent.consent,
                        acceptedProviderSchema: userConsent.provider_schema,
                        defaultConsent: providerSchemaToUserConsent(providers[idx].versions[0])
                    }

                    return acc
                }, {} as Record<ProviderId, { provider: Provider; userConsent: UserConsent | null; defaultConsent: UserConsent; acceptedProviderSchema?: ProviderSchema }>)

                this.providerIds = Object.keys(this.wizardUserConsents).map(Number)

                this.selectedProviderId = +Object.keys(this.wizardUserConsents)[0] ?? null
                this.loading = false
            })

            this.userConsents = this.providers.map((provider) => {
                const latestSchema = providerSchemaToUserConsent(provider.versions[0])
                return {
                    providerId: provider.id,
                    providerSchemaId: latestSchema.id,
                    verbs: extractVerbs(latestSchema, provider.id)
                }
            })
        })
    }

    next(): void {
        if (this.selectedProviderId) {
            const idx = this.providerIds.indexOf(this.selectedProviderId)
            if (idx + 1 < this.providerIds.length)
                this.selectedProviderId = this.providerIds[idx + 1]
        }
    }

    prev(): void {
        if (this.isSummary) {
            this.isSummary = false
            this.selectedProviderId = this.providerIds[this.providerIds.length - 1]
        } else {
            if (this.selectedProviderId) {
                const idx = this.providerIds.indexOf(this.selectedProviderId)
                if (idx - 1 >= 0) this.selectedProviderId = this.providerIds[idx - 1]
            }
        }
    }

    showSummary(): void {
        this.isSummary = true
    }

    changePrivacySetting(userConsent: UserConsent, providerId: number) {
        let consent = this.userConsents?.find(ele => ele.providerId == providerId)
        if(consent)
            consent.verbs = extractVerbs(userConsent, providerId)
        this.wizardUserConsents[providerId] = {
            ...this.wizardUserConsents[providerId],
            userConsent: userConsent
        }
    }

    submit(): void {
        this._apiService.saveUserConsent(this.userConsents).subscribe((response) => {
            this._toasterService.success($localize`:@@toastMessageSubmitted:Submitted`)
            this.dialogRef.close()
        })
    }

    logout(): void {
        this._authService.logout()
        this.dialogRef.close()
    }

    providerSchemaToUserConsent(providerSchema: ProviderSchema): UserConsent {
        return providerSchemaToUserConsent(providerSchema)
    }
}
